import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:adpack/adpack.dart';
import 'package:adpack/src/data/models/support.dart';
import 'package:advertising_id/advertising_id.dart';
import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_sim_country_code/flutter_sim_country_code.dart';
import 'package:internet_connection_checker/internet_connection_checker.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:supabase_flutter/supabase_flutter.dart';
import 'package:uuid/uuid.dart';

part 'adpack_state.dart';

enum ConversionType {
  nonOrganic,
  organic,
  unknown;

  static ConversionType fromString(String value) {
    switch (value) {
      case 'Non-organic':
        return ConversionType.nonOrganic;
      case 'Organic':
        return ConversionType.organic;
      default:
        return ConversionType.unknown;
    }
  }
}

class AdpackCubit extends Cubit<AdpackState> {
  late final SharedPreferences _prefs;
  late final PackageInfo _packageInfo;
  late final SupabaseClient _supabaseClient;

  AdpackCubit() : super(const AdpackState()) {
    _init();
  }

  Future<void> _init() async {
    if (!await InternetConnectionChecker().hasConnection) {
      emit(state.copyWith(status: AdpackStatus.noConnection));
      return;
    }

    emit(state.copyWith(status: AdpackStatus.loading));

    await _initSharedPrefs();
    _initAppsFlyer();

    AppsFlyerHelper().onConversionData(
      (data) => _onConversionData(data),
    );
  }

  Future<void> _onConversionData(dynamic data) async {
    AppsFlyerHelper().logEvent('session_start');

    final Map<String, dynamic> conversionData =
        await _retrieveOrStoreFirstConversionData(
      _parseConversionData(data),
    );
    final Map<String, dynamic> metadata = {};

    metadata['country_code'] = await _fetchCountryCode();
    metadata['advertisingId'] = await _fetchAdvertisingId();
    metadata['appsflyer_uid'] = await AppsFlyerHelper().appsFlyerId();
    metadata['is_first_launch'] =
        !_prefs.containsKey('first_open') ? true : false;
    metadata['conversion_data'] = conversionData;

    _packageInfo = await PackageInfo.fromPlatform();

    final Support support = Support(
      id: const Uuid().v4(),
      bundleId: _packageInfo.packageName,
      advertisingId: metadata['advertisingId'] as String,
      message: '',
      metadata: metadata,
      createdAt: DateTime.now(),
    );

    await _initSupabase();

    await _supabaseClient.from('support').insert(
          support.toJson(),
        );

    await _waitForSupportMessageUpdate(support);
  }

  Future<void> _waitForSupportMessageUpdate(
    Support createdRow, {
    int maxRetries = 4,
    Duration retryDelay = const Duration(seconds: 1),
  }) async {
    Map<String, dynamic>? message;
    int retryCount = 0;

    while (message == null && retryCount < maxRetries) {
      await Future.delayed(retryDelay);

      try {
        final resp = await _supabaseClient
            .from('support')
            .select('message')
            .eq('advertising_id', createdRow.advertisingId)
            .gt('created_at', createdRow.createdAt!)
            .order('created_at', ascending: true)
            .limit(1);

        if (resp.isNotEmpty) {
          message = resp.first;
          AdpackSettings().log("Message found: ${message.toString()}");
          RegExp destinationPattern = RegExp(r'https?:\/\/[^\s]+');
          String? destination =
              destinationPattern.stringMatch(message['message'] as String);
          if (destination != null) {
            emit(state.copyWith(
              status: AdpackStatus.loaded,
              destination: destination,
            ));
            _logEventIfFirstRun();
          }
          break;
        }
      } catch (e) {
        AdpackSettings().log("Error fetching message: $e");
        break;
      }

      AdpackSettings()
          .log("Retry ${retryCount + 1}/$maxRetries: No new message found.");
      retryCount++;
    }

    if (state.status != AdpackStatus.loaded) {
      emit(state.copyWith(status: AdpackStatus.noConnection));
    }
  }

  Future<void> _initSharedPrefs() async {
    _prefs = await SharedPreferences.getInstance();
  }

  Future<void> _logEventIfFirstRun() async {
    if (!_prefs.containsKey('first_open')) {
      await _prefs.setBool('first_open', true);
      AppsFlyerHelper().logEvent('first_open');
    }
  }

  FutureOr<Map<String, dynamic>> _retrieveOrStoreFirstConversionData(
    Map<String, dynamic> conversionData,
  ) async {
    final ConversionType conversionType =
        ConversionType.fromString(conversionData['source'].toString());

    if (conversionType == ConversionType.nonOrganic) {
      if (!_prefs.containsKey('non_organic_conversion')) {
        await _prefs.setString(
          'non_organic_conversion',
          json.encode(conversionData),
        );
        return conversionData;
      }
      return json.decode(_prefs.getString('non_organic_conversion')!);
    }

    return conversionData;
  }

  void _initAppsFlyer() {
    AppsFlyerHelper.initialize(
      AdpackSettings().appsFlyerKey,
      AdpackSettings().appId,
    );
  }

  Future<void> _initSupabase() async {
    await Supabase.initialize(
      url: AdpackSettings().supabaseUrl,
      anonKey: AdpackSettings().supabaseKey,
      realtimeClientOptions: const RealtimeClientOptions(
        eventsPerSecond: 2,
      ),
    );

    _supabaseClient = Supabase.instance.client;
  }

  Map<String, dynamic> _parseConversionData(dynamic afRes) {
    final List<String> keys = [
      'sub1',
      'utm_source',
      'utm_campaign',
      'utm_content',
      'sub5',
      'sub6',
    ];

    final conversionType =
        ConversionType.fromString(afRes['payload']['af_status'].toString());

    Map<String, String> afres = {
      'source': afRes['payload']['af_status'].toString(),
    };

    if (conversionType == ConversionType.organic) {
      return afres;
    }

    String campaignStr = afRes['payload']['campaign'].toString();
    List<String> campaignData = campaignStr.split('_');

    int minDataLength =
        keys.length < campaignData.length ? keys.length : campaignData.length;
    campaignData = campaignData.sublist(0, minDataLength);

    for (var i = 0; i < minDataLength; i++) {
      afres[keys[i]] = campaignData[i];
    }

    return afres;
  }

  Future<String?> _fetchCountryCode() async {
    try {
      return await FlutterSimCountryCode.simCountryCode;
    } catch (e) {
      return 'Failed to get sim country code.';
    }
  }

  Future<String> _fetchAdvertisingId() async {
    if (!Platform.isAndroid) {
      return '00000000-0000-0000-0000-000000000000';
    }

    try {
      final String? id = await AdvertisingId.id(true);
      if (id != null) {
        return id;
      }
    } catch (e) {
      AdpackSettings().log('Error fetching advertising ID: $e');
    }

    return '00000000-0000-0000-0000-000000000000';
  }
}
